"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheHelper = void 0;
const index_1 = require("../index");
let CacheHelper = /** @class */ (() => {
    class CacheHelper {
        static getInfo() {
            let cacheList = [];
            if (this.cacheList !== null) {
                this.cacheList.forEach((item) => {
                    const cache = {};
                    cache.name = item.name;
                    cache.isExpired = item.cacheExpireDate.getTime() < new Date().getTime();
                    cache.minutesToLive = ((item.millisecondsToLive / 60) / 1000);
                    cacheList.push(cache);
                });
            }
            return cacheList;
        }
        static get(name) {
            const cache = this.cacheList.get(name);
            if (cache) {
                if (cache.cacheExpireDate.getTime() > new Date().getTime()) {
                    return cache.data;
                }
            }
            return null;
        }
        static set(name, data, minutesToLive = 1) {
            this.cacheList.delete(name);
            const cacheItem = new CacheItem(name, data, minutesToLive);
            this.cacheList.set(name, cacheItem);
            index_1.logger.debug(`Created cache: ${name}`, 'ihub.framework.cache.set');
        }
    }
    CacheHelper.cacheList = new Map();
    return CacheHelper;
})();
exports.CacheHelper = CacheHelper;
class CacheItem {
    constructor(name, data, minutesToLive) {
        this.name = name;
        this.data = data;
        this.millisecondsToLive = minutesToLive * 60 * 1000;
        this.fetchDate = new Date();
        this.cacheExpireDate = new Date(this.fetchDate.getTime() + this.millisecondsToLive);
    }
}
//# sourceMappingURL=index.js.map