"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cron = void 0;
const cron_1 = __importDefault(require("cron"));
const uniqid = require('uniqid');
const index_1 = require("../../index");
__exportStar(require("./CronModel"), exports);
const CronJob = cron_1.default.CronJob;
class Cron {
    constructor(routines, model) {
        this.crons = [];
        this.model = model;
        // Loads the routine
        routines.forEach((job) => __awaiter(this, void 0, void 0, function* () {
            index_1.logger.debug(`Loading routine ${job.id}`, 'ihf.component.routines.loading');
            // Check if it can't run at the same time
            if (job.concurrently === false) {
                const cronExists = yield this.getCronControl(job.id);
                // If there isn't any control doc we create it
                if (!cronExists) {
                    try {
                        yield new model({
                            jobId: job.id,
                        }).save();
                    }
                    catch (e) {
                        // TODO think what to to in case of error
                        // console.log(e)
                    }
                }
            }
            // Lets give this job a unique id
            const runningId = uniqid();
            const cronJob = new CronJob(job.cron, () => __awaiter(this, void 0, void 0, function* () {
                index_1.logger.info(`Job for ${job.id} started`, `ihf.component.routines.${job.id}.started`);
                let apmTransaction = false;
                if (process.env.APM_ENABLED === 'true')
                    apmTransaction = this.transStart(job.id);
                // Update the control doc and get the non update return to see if it was already running
                const start = yield this.cronControlStart(job.id, runningId);
                // Lets check the sla for any error
                const sla = job.sla || 300000;
                const cronControlRawDoc = yield this.getCronControl(job.id);
                if (cronControlRawDoc.running) {
                    const slaMSDelta = new Date().getTime() - new Date(cronControlRawDoc.startDate).getTime();
                    if (slaMSDelta > sla) {
                        index_1.logger.error(`Job for ${job.id} SLA error`, `ihf.component.routines.${job.id}.error`);
                        if (apmTransaction)
                            this.transEnd(apmTransaction);
                        return;
                    }
                }
                if (!start) {
                    index_1.logger.debug(`Job for ${job.id} skipped`, `ihf.component.routines.${job.id}.skipped`);
                    if (apmTransaction)
                        this.transEnd(apmTransaction);
                    return;
                }
                // Do the routine action
                job.action(() => __awaiter(this, void 0, void 0, function* () {
                    const finish = yield this.cronControlFinish(job.id, runningId);
                    if (apmTransaction)
                        this.transEnd(apmTransaction);
                    if (finish.jobRunningId !== runningId) {
                        index_1.logger.error('Job conc error', 'ihf.component.routines.conc.error');
                    }
                    index_1.logger.info(`Job for ${job.id} done`, `ihf.component.routines.${job.id}.done`);
                }));
            }), null, true, 'America/Sao_Paulo', null, job.runOnInit);
            cronJob.runningId = runningId;
            index_1.counter.crons.push(cronJob);
        }));
    }
    cronControlStart(id, runningId) {
        return __awaiter(this, void 0, void 0, function* () {
            const update = {
                running: true,
                startDate: Date.now(),
                jobRunningId: runningId,
                finishedDate: '',
            };
            return yield this.model.findOneAndUpdate({
                jobId: id,
                running: false,
            }, update, {
                new: true
            });
        });
    }
    cronControlFinish(id, runningId) {
        return __awaiter(this, void 0, void 0, function* () {
            const update = {
                running: false,
                startDate: Date.now(),
                finishedDate: Date.now(),
            };
            return yield this.model.findOneAndUpdate({
                jobId: id,
                jobRunningId: runningId,
                running: true,
            }, update, {
                new: true
            });
        });
    }
    getCronControl(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.model.findOne({
                jobId: id,
            });
        });
    }
    transStart(id) {
        let apmTransaction = null;
        if (index_1.apm)
            apmTransaction = index_1.apm.startTransaction(id, 'Routines');
        return apmTransaction;
    }
    transEnd(trans) {
        return __awaiter(this, void 0, void 0, function* () {
            if (trans)
                trans.end();
        });
    }
}
exports.Cron = Cron;
//# sourceMappingURL=index.js.map